package com.diagbot.web;

import com.diagbot.dto.FileDTO;
import com.diagbot.dto.FileThumDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.UploadFacade;
import com.diagbot.util.FastJsonUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

@RestController
@Api(value = "文件上传API", tags = { "智能预问诊-文件上传API" })
@RequestMapping(value = "/file")
@SuppressWarnings("unchecked")
public class UploadController {
    @Autowired
    private UploadFacade uploadFacade;

    @ApiOperation(value = "智能预问诊-单个文件上传")
    @CrossOrigin(allowCredentials = "true", allowedHeaders = "*", methods = { RequestMethod.POST }, origins = "*")
    @PostMapping(value = "/uploadImage", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public RespDTO<FileDTO> singleFileUpload(@RequestParam("upfile") MultipartFile file) {
        FileDTO data = uploadFacade.singleFileUpload(file);
        if (data.getState().equals("FAILURE")) {
            return RespDTO.onError(data.getInfo());
        } else {
            return RespDTO.onSuc(data);
        }
    }

    @ApiOperation(value = "智能预问诊-单个文件上传同时生成缩略图")
    @CrossOrigin(allowCredentials = "true", allowedHeaders = "*", methods = { RequestMethod.POST }, origins = "*")
    @PostMapping(value = "/uploadImageThum", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public RespDTO<FileThumDTO> singleFileThumUpload(@RequestParam("upfile") MultipartFile file) {
        FileThumDTO data = uploadFacade.singleFileThumUpload(file);
        if (null != data.getSource() &&
                data.getSource().getState().equals("FAILURE")) {
            return RespDTO.onError(data.getSource().getInfo());
        } else {
            return RespDTO.onSuc(data);
        }
    }

    @ApiOperation(value = "智能预问诊-多个文件上传")
    @CrossOrigin(allowCredentials = "true", allowedHeaders = "*", methods = { RequestMethod.POST }, origins = "*")
    @PostMapping(value = "/uploadImages", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public RespDTO<List<FileDTO>> multiFileUpload(@RequestParam("upfiles") MultipartFile[] file) {
        List<FileDTO> data = uploadFacade.multiFileUpload(file);
        String msg = "";
        if (ListUtil.isNotEmpty(data)) {
            for (int i = 0; i < data.size(); i++) {
                if (data.get(i).getState().equals("FAILURE")) {
                    msg += "第【" + (i + 1) + "】张图片上传失败，" + data.get(i).getInfo() + "；";
                }
            }
        }
        if (StringUtil.isNotBlank(msg)) {
            return RespDTO.onError(msg);
        } else {
            return RespDTO.onSuc(data);
        }
    }

    @ApiOperation(value = "智能预问诊-多个文件上传同时生成缩略图")
    @CrossOrigin(allowCredentials = "true", allowedHeaders = "*", methods = { RequestMethod.POST }, origins = "*")
    @PostMapping(value = "/uploadImageThums", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public RespDTO<List<FileThumDTO>> multiFileThumUpload(@RequestParam("upfiles") MultipartFile[] file, @RequestParam("type") Integer[] type) {
        List<FileThumDTO> data = uploadFacade.multiFileThumUpload(file, type);
        String msg = "";
        if (ListUtil.isNotEmpty(data)) {
            for (int i = 0; i < data.size(); i++) {
                if (data.get(i).getSource() != null && data.get(i).getSource().getState().equals("FAILURE")) {
                    msg += "第【" + (i + 1) + "】张图片上传失败，" + data.get(i).getSource().getInfo() + "；";
                }
                if (data.get(i).getThumbnail() != null && data.get(i).getThumbnail().getState().equals("FAILURE")) {
                    msg += "第【" + (i + 1) + "】张图片缩略图上传失败，" + data.get(i).getThumbnail().getInfo() + "；";
                }
            }
        }
        if (StringUtil.isNotBlank(msg)) {
            return RespDTO.onError(msg);
        } else {
            return RespDTO.onSuc(data);
        }
    }

    @PostMapping("/deleteRemoteFile")
    @ApiOperation(value = "智能预问诊-文件删除")
    public String deleteRemoteFile(@RequestParam("path") String path) {
        String data = FastJsonUtils.getBeanToJson(uploadFacade.deleteRemoteFile(path));
        return data;
    }
}